# -*- python -*-
# $RCSfile: boundary_condition_test.py,v $
# $Revision: 1.6 $
# $Author: langer $
# $Date: 2008-06-12 20:56:58 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# Tests for the machinery that handles boundary condition
# intersections, floatBC promotion, and enabling and disabling of
# boundary conditions.

import unittest, os

# Define a decorator so that all test methods check that Skeleton and
# Microstructure objects are deallocated properly.  These tests have
# to be done after the Microstructure is deleted, but they can't be
# put in the tearDown method because UnitTest.assertXXXX doesn't work
# there.  The tearDown operations are done here, instead.
def checkCCounts(*_microstructures):
    def decorator(func):
        def checktest(self, *args, **kwargs):
            val = func(self, *args, **kwargs)
            for msname in _microstructures:
                OOF.Microstructure.Delete(microstructure=msname)
#             self.assertEqual(femesh.get_globalFEMeshCount(), 0)
#             self.assertEqual(cskeleton.get_globalNodeCount(), 0)
#             self.assertEqual(cskeleton.get_globalElementCount(), 0)
#             self.assertEqual(cmicrostructure.get_globalMicrostructureCount(), 0)
            return val
        return checktest
    return decorator

class OOF_BCTest(unittest.TestCase):
    def setUp(self):
        global femesh, cskeleton, cmicrostructure
        from ooflib.SWIG.common import cmicrostructure
        from ooflib.SWIG.engine import cskeleton, femesh
        OOF.File.Load.Data(
            filename=os.path.join("bc_data", "periodic_test_mesh"))

    @checkCCounts("saved_data", "bc_test")
    def SimpleSolve(self):
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
        OOF.File.Load.Data(filename=os.path.join("bc_data",
                                                 "simple_solution_test"))
        from ooflib.engine import mesh
        saved = mesh.meshes["saved_data:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
 

    # periodic in one direction
    @checkCCounts("periodic_1", "bc_test")
    def Periodic1(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=PeriodicBC(field=Temperature,field_component='',
                                 equation=Heat_Eqn,eqn_component='',
                                 boundary='top bottom'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='periodic_1')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "periodic_1"),
##                            mode="w",format="ascii",mesh="periodic_1:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='periodic_1', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "periodic_1"))
        from ooflib.engine import mesh
        saved = mesh.meshes["periodic_1:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # periodic in two directions
    @checkCCounts("periodic_2", "bc_test")
    def Periodic2(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=PeriodicBC(field=Temperature,field_component='',
                                 equation=Heat_Eqn,eqn_component='',
                                 boundary='top bottom'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=PeriodicBC(field=Temperature,field_component='',
                                 equation=Heat_Eqn,eqn_component='',
                                 boundary='left right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='periodic_2')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "periodic_2"),
##                            mode="w",format="ascii",mesh="periodic_2:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='periodic_2', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "periodic_2"))
        from ooflib.engine import mesh
        saved = mesh.meshes["periodic_2:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # float intersecting float
    @checkCCounts("twofloats", "bc_test")
    def TwoFloats(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='twofloats')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "twofloats"),
##                            mode="w",format="ascii",mesh="twofloats:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='twofloats', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofloats"))
        from ooflib.engine import mesh
        saved = mesh.meshes["twofloats:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # loop of floats
    @checkCCounts("floatloop", "bc_test")
    def FloatLoop(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<5>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<6>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='-alpha'),
                              boundary='right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='floatloop')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "floatloop"),
##                            mode="w",format="ascii",mesh="floatloop:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='floatloop', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "floatloop"))
        from ooflib.engine import mesh
        saved = mesh.meshes["floatloop:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # inconsistent loop of floats
    @checkCCounts("twofloats", "bc_test")
    def InconsistentFloatLoop(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<5>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<6>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='right'))
        from ooflib.SWIG.common import ooferror
        # make sure an inconsistent loop raises an error
        self.assertRaises(ooferror.ErrSetupError,
                          OOF.Solver.Solve,mesh="bc_test:skeleton:mesh",
                          solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                inplane=ILUPreconditioner()))))
        # now disable two of the boundaries to make it consistent (and
        # so that it matches the test in TwoFloats)
        OOF.Mesh.Boundary_Conditions.Disable(mesh='bc_test:skeleton:mesh',
                                             name='bc<6>')
        OOF.Mesh.Boundary_Conditions.Disable(mesh='bc_test:skeleton:mesh',
                                             name='bc<5>')
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofloats"))
        from ooflib.engine import mesh
        saved = mesh.meshes["twofloats:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        # now re-enable the boundaries and make sure it raises the error again
        OOF.Mesh.Boundary_Conditions.Enable(mesh='bc_test:skeleton:mesh',
                                            name='bc<6>')
        OOF.Mesh.Boundary_Conditions.Enable(mesh='bc_test:skeleton:mesh',
                                            name='bc<5>')
        self.assertRaises(ooferror.ErrSetupError,
            OOF.Solver.Solve,mesh="bc_test:skeleton:mesh",
                             solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                inplane=ILUPreconditioner()))))


    # float intersecting fixed
    @checkCCounts("floatandfixed", "bc_test")
    def FloatAndFixed(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=DirichletBC(field=Temperature,field_component='',
                                  equation=Heat_Eqn,eqn_component='',
                                  profile=ContinuumProfile(function='0'),
                                  boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='floatandfixed')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "floatandfixed"),
##                            mode="w",format="ascii",mesh="floatandfixed:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='floatandfixed', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "floatandfixed"))
        from ooflib.engine import mesh
        saved = mesh.meshes["floatandfixed:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # loop of floats
    @checkCCounts("fixedfloatloop", "bc_test")
    def FloatAndFixedLoop(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=DirichletBC(field=Temperature,field_component='',
                                  equation=Heat_Eqn,eqn_component='',
                                  profile=ContinuumProfile(function='0'),
                                  boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),
                              boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<5>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),
                              boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<6>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='-alpha'),
                              boundary='right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='fixedfloatloop')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "fixedfloatloop"),
##                            mode="w",format="ascii",mesh="fixedfloatloop:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='fixedfloatloop', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "fixedfloatloop"))
        from ooflib.engine import mesh
        saved = mesh.meshes["fixedfloatloop:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)


    # fixed intersecting fixed
    @checkCCounts("twofixed", "bc_test")
    def TwoFixed(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=DirichletBC(field=Temperature,field_component='',
                                  equation=Heat_Eqn,eqn_component='',
                                  profile=ContinuumProfile(function='0'),
                                  boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=DirichletBC(field=Temperature,field_component='',
                                  equation=Heat_Eqn,eqn_component='',
                                  profile=ContinuumProfile(function='alpha'),
                                  boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                        inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='twofixed')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "twofixed"),
##                            mode="w",format="ascii",mesh="twofixed:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='twofixed', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofixed"))
        from ooflib.engine import mesh
        saved = mesh.meshes["twofixed:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)

        
    def tearDown(self):
        OOF.Property.Delete(property='Color:blue')
        OOF.Property.Delete(property='Color:white')
        OOF.Property.Delete(property='Thermal:Conductivity:Isotropic:blue_k')
        OOF.Property.Delete(property='Thermal:Conductivity:Isotropic:white_k')
        OOF.Property.Delete(property='Mechanical:Elasticity:Isotropic:blue_elastic')
        OOF.Property.Delete(property='Mechanical:Elasticity:Isotropic:white_elastic')
        OOF.Material.Delete(name='bulk')
        OOF.Material.Delete(name='boundaries')



# Routine to do regression-type testing on the items in this file.
# Tests must be run in the order they appear in the list.  This
# routine will stop after the first failure.
def run_tests():

    test_set = [
        OOF_BCTest("SimpleSolve"),
        OOF_BCTest("Periodic1"),
        OOF_BCTest("Periodic2"),
        OOF_BCTest("TwoFloats"),
        OOF_BCTest("FloatLoop"),
        OOF_BCTest("InconsistentFloatLoop"),
        OOF_BCTest("FloatAndFixed"),
        OOF_BCTest("FloatAndFixedLoop"),
        OOF_BCTest("TwoFixed")
        ]
    
    logan = unittest.TextTestRunner()
    for t in test_set:
        print "\n *** Running test: %s\n" % t.id()
        res = logan.run(t)
        if not res.wasSuccessful():
            return 0
    return 1


###################################################################
# The code below this line should be common to all testing files. #
###################################################################

if __name__=="__main__":
    # If directly run, then start oof, and run the local tests, then quit.
    import sys
    try:
        import oof2
        sys.path.append(os.path.dirname(oof2.__file__))
        from ooflib.common import oof
    except ImportError:
        print "OOF is not correctly installed on this system."
        sys.exit(4)
    sys.argv.append("--text")
    sys.argv.append("--quiet")
    sys.argv.append("--seed=17")
    oof.run(no_interp=1)
    
    success = run_tests()

    OOF.File.Quit()
    
    if success:
        print "All tests passed."
    else:
        print "Test failure."
